package com.xkrs.service.impl;

import com.xkrs.common.encapsulation.PromptMessageEnum;
import com.xkrs.dao.FirePointOrdinaryDao;
import com.xkrs.dao.SysUserDao;
import com.xkrs.helper.FirePointQueryManager;
import com.xkrs.model.bean.FirePointQueryResultBean;
import com.xkrs.model.entity.FirePointOrdinaryEntity;
import com.xkrs.model.entity.SysUserEntity;
import com.xkrs.model.vo.AllFirePointVo;
import com.xkrs.service.FirePointService;
import com.xkrs.utilsnew.DateTimeUtils;
import com.xkrs.utilsnew.FirePointConvertUtils;
import com.xkrs.utilsnew.TokenUtil;
import org.apache.hc.core5.util.TextUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import static com.xkrs.common.encapsulation.OutputEncapsulation.outputEncapsulationObject;

@CacheConfig(cacheNames = "FirePointServiceCache")
@Service
public class FirePointServiceImpl implements FirePointService {

    public static Logger log = LoggerFactory.getLogger(FirePointServiceImpl.class);

    private final Locale locale = LocaleContextHolder.getLocale();

    @Resource
    private SysUserDao sysUserDao;

    @Resource
    private FirePointQueryManager firePointQueryManager;

    @Resource
    private FirePointOrdinaryDao firePointOrdinaryDao;

    /**
     * 查询火点播报（无token）
     *
     * @param limit 最大数量
     */
    @Override
    public String queryFirePointBroadcast(Integer limit) {
        FirePointQueryResultBean firePointQueryResult = firePointQueryManager.queryFirePointBroadcast(limit);
        if (firePointQueryResult.isSuccess()) {
            return outputEncapsulationObject(PromptMessageEnum.SUCCESS, firePointQueryResult.getList(), locale);
        } else {
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL, firePointQueryResult.getErrorMessage(), locale);
        }
    }

    /**
     * 动态多条件查询
     */
    @Override
    public String queryFirePoint(String token, String code, String startTime, String endTime, String satelliteType, String landType) {
        //验证token
        String userName = TokenUtil.obtainUserNameByToken(token);
        if (TextUtils.isEmpty(userName)) {
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL, "账号不存在或错误", locale);
        }
        SysUserEntity sysUserEntity = sysUserDao.selectByUserName(userName);
        if (sysUserEntity == null) {
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL, "账号不存在或错误", locale);
        }

        FirePointQueryResultBean firePointQueryResult = firePointQueryManager.queryFirePointBelongToUser(sysUserEntity, code, startTime, endTime, satelliteType, landType);
        if (firePointQueryResult.isSuccess()) {
            return outputEncapsulationObject(PromptMessageEnum.SUCCESS, firePointQueryResult.getList(), locale);
        } else {
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL, firePointQueryResult.getErrorMessage(), locale);
        }
    }

    /**
     * 查询今天的火点信息
     */
    @Override
    public String selectTodayFirePoint(String token, String countyCode) {

        //验证token
        String userName = TokenUtil.obtainUserNameByToken(token);
        if (TextUtils.isEmpty(userName)) {
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL, "账号不存在或错误", locale);
        }
        SysUserEntity sysUserEntity = sysUserDao.selectByUserName(userName);
        if (sysUserEntity == null) {
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL, "账号不存在或错误", locale);
        }

        LocalDateTime startTime1 = LocalDateTime.now();
        LocalDateTime startTime2 = LocalDateTime.of(startTime1.getYear(), startTime1.getMonth(), startTime1.getDayOfMonth(), 0, 0, 0, 0);
        String startTimeString = DateTimeUtils.localDateTimeToString(startTime2);

        LocalDateTime endTime1 = LocalDateTime.now().plusDays(1);
        LocalDateTime endTime2 = LocalDateTime.of(endTime1.getYear(), endTime1.getMonth(), endTime1.getDayOfMonth(), 0, 0, 0, 0);
        String endTimeString = DateTimeUtils.localDateTimeToString(endTime2);

        FirePointQueryResultBean firePointQueryResult = firePointQueryManager.queryFirePointBelongToUser(sysUserEntity, countyCode, startTimeString, endTimeString, null, null);
        if (firePointQueryResult.isSuccess()) {
            return outputEncapsulationObject(PromptMessageEnum.SUCCESS, firePointQueryResult.getList(), locale);
        } else {
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL, firePointQueryResult.getErrorMessage(), locale);
        }
    }

    /**
     * 根据火点编码查询火点信息
     */
    @Override
    public String selectFirePointByCode(String fireCode) {
        if (TextUtils.isEmpty(fireCode)) {
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL, null, locale);
        }
        Specification<FirePointOrdinaryEntity> specification = (root, criteriaQuery, criteriaBuilder) -> {
            //查询条件集合
            List<Predicate> predicateList = new ArrayList<>();
            predicateList.add(criteriaBuilder.equal(root.get("fireCode").as(String.class), fireCode));
            Predicate[] predicateArray = new Predicate[predicateList.size()];
            return criteriaBuilder.and(predicateList.toArray(predicateArray));
        };
        List<FirePointOrdinaryEntity> firePointOrdinaryList = firePointOrdinaryDao.findAll(specification, Sort.by(Sort.Direction.DESC, "satelliteTime"));
        if (firePointOrdinaryList.size() == 1) {
            AllFirePointVo firePointVo = FirePointConvertUtils.convert(firePointOrdinaryList.get(0));
            return outputEncapsulationObject(PromptMessageEnum.SUCCESS, firePointVo, locale);
        }
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, null, locale);
    }

}
